from __future__ import unicode_literals

import threading
import time

import zmq
from om.message import Message


class ZMQTestMixin(object):
    ZMQ_SUB_PORT = 10100
    ZMQ_PUB_PORT = 10200
    _command_topic = b"_testcase_commands"
    ZMQ_STARTUP_SLEEP = 1
    topics = []

    def _start_broker(self):
        def broker():
            alpr_endpoint = self.zmq_context.socket(zmq.XSUB)
            service_endpoint = self.zmq_context.socket(zmq.XPUB)
            alpr_endpoint.setsockopt(zmq.LINGER, 0)
            service_endpoint.setsockopt(zmq.LINGER, 0)
            alpr_endpoint.bind("tcp://*:{}".format(self.ZMQ_SUB_PORT))
            service_endpoint.bind("tcp://*:{}".format(self.ZMQ_PUB_PORT))
            try:
                zmq.proxy(alpr_endpoint, service_endpoint)
            except zmq.ContextTerminated:
                pass
            alpr_endpoint.close()
            service_endpoint.close()

        thread = threading.Thread(target=broker)
        thread.start()
        return thread

    def _zmq_sub_recvloop(self):
        while True:
            topic, message = Message.from_socket(self.zmq_sub)
            if topic == self._command_topic:
                if message["cmd"] == "hi":
                    self.zmq_connected.set()
                elif message["cmd"] == "exit":
                    break
            else:
                with self.zmq_recv_lock:
                    self.zmq_recv_messages.append((topic, message))
                    self.zmq_has_messages.set()

    def _start_zmq(self):
        self.zmq_connected = threading.Event()
        self.zmq_context = zmq.Context()
        self.zmq_broker_thread = self._start_broker()
        self.zmq_pub = self.zmq_context.socket(zmq.PUB)
        self.zmq_pub.setsockopt(zmq.LINGER, 0)
        self.zmq_pub.connect("tcp://127.0.0.1:10100")
        self.zmq_sub = self.zmq_context.socket(zmq.SUB)
        self.zmq_sub.setsockopt(zmq.LINGER, 0)
        self.zmq_sub.connect("tcp://127.0.0.1:10200")
        self.zmq_sub.subscribe(self._command_topic)
        self.zmq_sub_thread = threading.Thread(target=self._zmq_sub_recvloop)
        self.zmq_sub_thread.start()
        self.zmq_recv_lock = threading.Lock()
        self.zmq_recv_messages = []
        self.zmq_has_messages = threading.Event()

        for x in range(20):
            self._zmq_cmd("hi")
            self.zmq_connected.wait(timeout=0.05)
        assert self.zmq_connected.is_set(), "broker startup and connect failed"

        time.sleep(self.ZMQ_STARTUP_SLEEP)
        for topic in self.topics:
            self.zmq_subscribe(topic)

    def _zmq_cmd(self, cmd):
        Message(cmd=cmd).to_socket(self.zmq_pub, self._command_topic)

    def _stop_zmq(self):
        self._zmq_cmd("exit")
        self.zmq_sub_thread.join()
        self.zmq_sub.close()
        self.zmq_pub.close()
        self.zmq_context.term()
        self.zmq_broker_thread.join()

    def zmq_subscribe(self, topic):
        self.zmq_sub.subscribe(topic)

    def zmq_send(self, topic, _message=None, **kwargs):
        data = _message or kwargs
        assert not (_message and kwargs), "pass the message dict as the first parameter, or use the kwargs, not both"
        assert "type" in data, "a message must always have a type"
        if "name" not in data:
            data["name"] = "testcase"
        Message(**data).to_socket(self.zmq_pub, topic)

    def _zmq_pop_from_front(self):
        with self.zmq_recv_lock:
            msg = self.zmq_recv_messages.pop(0)
            if not self.zmq_recv_messages:
                self.zmq_has_messages.clear()
            return msg

    def zmq_wait_for_message(self, topic=None, timeout=1):
        start = time.time()
        while True:
            if not self.zmq_has_messages.wait(timeout=timeout):
                raise TimeoutError()

            while self.zmq_recv_messages:
                message_topic, message = self._zmq_pop_from_front()
                if not topic or message_topic == topic:
                    return topic, message

            time_diff = time.time() - start
            if time_diff > timeout:
                raise TimeoutError()

    def setUp(self):
        super(ZMQTestMixin, self).setUp()
        self._start_zmq()

    def tearDown(self):
        super(ZMQTestMixin, self).tearDown()
        self._stop_zmq()



import fnmatch
import six


class Filter(dict):
    _RESERVED_KEYS = [
        "_name", "topic",
    ]
    RESERVED_KEYS = []
    REQUIRED_KEYS = []

    def __init__(self, **data):
        super(Filter, self).__init__(**data)
        for key in self.REQUIRED_KEYS:
            assert key in self

    def __eq__(self, other):
        return self.name == other.name

    def _strip_wildcard(self, topic):
        for char in ["*", "?", "[", "]"]:
            topic = topic.replace(char, "")
        return topic

    def _match_dict(self, message, data):
        for filter_key, filter_value in data.items():
            if filter_key in self._RESERVED_KEYS + self.RESERVED_KEYS:
                continue

            inverted = isinstance(filter_value, (six.string_types)) and filter_value[0] == "!"
            if inverted:
                filter_value = filter_value[1:]

            message_value = message.get(filter_key)

            if isinstance(filter_value, dict):
                if not self._match_dict(message_value, filter_value):
                    return False
            elif isinstance(filter_value, (list, tuple)):
                if message_value not in filter_value:
                    return False
            elif inverted and message_value == filter_value:
                return False
            elif message_value != filter_value:
                return False
        return True

    def subscribe(self, socket):
        if "topic" in self:
            socket.subscribe(self._strip_wildcard(self["topic"]).encode("ascii"))

    def match(self, topic, message):
        if hasattr(topic, "decode"):
            topic = topic.decode("ascii")
        if "topic" in self and (topic == self["topic"] or fnmatch.fnmatch(topic, self["topic"])):
            return self._match_dict(message, self)
        else:
            return False

    @property
    def name(self):
        return self.get("_name") or self.get("name")

    @property
    def topic(self):
        return self.get("topic")



import glob
import json
import logging
import os
from dotenv import dotenv_values
import yaml


def load_hw_env(path):
    env_dict = dotenv_values(path)
    try:
        env_dict["HARDWARE_TYPE"] = json.loads(env_dict["HARDWARE_TYPE"])
    except:
        pass
    return env_dict


def get_hw_packages(type, package_dir="/opt/openmodule/dist2/"):
    """get hardware packages matching type with prefix match, so it's possible to match as specific as necessary

    Args:
        type (str): prefix to match, e.g. "io" for all io packages or "cam-ip-avigilon" for all avigilon cameras
        package_dir (str): path to directory containing the hardware packages
    Returns:
        list of directories of the matching hardware packages
    """
    all_dirs = sorted(glob.glob(os.path.join(package_dir, "hw-*")))
    all_dirs = [d for d in all_dirs if os.path.isdir(d) and os.path.exists(os.path.join(d, "env"))]
    matching_dirs = []
    for d in all_dirs:
        env = os.path.join(d, "env")
        try:
            env_dict = load_hw_env(env)
            for hw_type in env_dict.get("HARDWARE_TYPE", []):
                if hw_type.startswith(type):
                    matching_dirs.append(d)
                    break
        except Exception as e:
            logging.error("Unexpected error while loading hardware env file " + env)
    return matching_dirs


def get_hw_settings(type, package_dir="/opt/openmodule/dist2/"):
    """get settings of the hardware packages prefix-matching type

    Args:
        type (str): prefix to match, e.g. "io" for all io packages or "cam-ip-avigilon" for all avigilon cameras
        package_dir (str): path to directory containing the hardware packages
    Returns:
        list of {"env": dict, "yml": [dict,list,None]}, where in "env" one can find the env variables as a dict and in
            "yml" one can find the yaml settings
    """
    hw_dirs = get_hw_packages(type, package_dir)
    hw_settings = []
    for d in hw_dirs:
        env = os.path.join(d, "env")
        yml = os.path.join(d, "yml")
        try:
            hw_settings.append({"env": load_hw_env(env), "yml": {}})
        except:
            logging.error("Unexpected error while loading hardware env file " + env)
        if os.path.exists(os.path.join(d, "yml")):
            try:
                with open(yml) as f:
                    hw_settings[-1]["yml"] = yaml.load(f, Loader=yaml.FullLoader)
            except:
                logging.error("Failed to load yml " + yml)
    return hw_settings


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    print("1", get_hw_packages("io"))
    print("2", get_hw_settings("cam"))
    print("3", get_hw_settings("cam", "/tmp"))




# coding=utf-8
from __future__ import unicode_literals

import zmq

from om.message import Topic, Message


class Status:
    OK = "ok"
    ERROR = "error"
    OFFLINE = "offline"


class AlertHandler(object):
    def __init__(self, config=None, context=None, pub_socket=None, name=None, version=None):
        self._pub_socket = None
        self._alerts = []

        if config:
            self._version = config.VERSION
            self._name = config.NAME
        else:
            self._version = version
            self._name = name

        if pub_socket:
            self._pub_socket = pub_socket
        else:
            context = context or zmq.Context()
            assert config, "a config must be set to auto-connect to the broker!"
            self._pub_socket = context.socket(zmq.PUB)
            self._pub_socket.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_SUB_PORT))

    def _send_message(self, alert_id, _status_dict):
        alert = self._alerts[alert_id]
        _status_dict["alert_meta"].update(dict(sender=self._name))
        old_status = alert["alert_dict"].get("status")
        alert["alert_dict"].update(_status_dict)
        if alert["always_send_alert"] or alert["alert_dict"]["status"] != old_status:
            message = Message(
                name=self._name,
                type="alert",
            )
            message.update(alert["alert_dict"])
            message.to_socket(self._pub_socket, Topic.ALERT)
            return True
        return False

    def get_or_add_alert_id(self, source, alert_type, handle_type, package_name=None, version=None,
                            always_send_alert=None):
        """Define an alert with its name parameters

        Args:
            source (str): The source of the error (i.e. which pin, camera)
            alert_type (str): Name of the alert all lower_case with '_' (i.e. high_time_check)
            handle_type (str): How the error should be handled on the server (state, state_change, count)
            package_name (str) (default=config.NAME): You send the error for this package
            version (str) (default=config.VERSION): Current version of the package
            always_send_alert (bool) (default=False): Send every alert or only when there is a change in the status

        Returns:
            alert_id of the saved alert
        """

        assert "-" not in alert_type, "please do not use - in your alert type names"

        alert_dict = dict(source=source, alert_type=alert_type, handle_type=handle_type)
        alert_dict["package"] = package_name or self._name

        for alert in self._alerts:
            if alert_dict.items() <= alert["alert_dict"].items():
                if always_send_alert is not None:
                    alert["always_send_alert"] = always_send_alert
                if version is not None:
                    alert["alert_dict"]["version"] = version
                return alert["id"]

        v = version if version is not None else self._version
        asa = always_send_alert if always_send_alert is not None else False
        alert = {"alert_dict": alert_dict, "id": len(self._alerts), "always_send_alert": asa, "version": v}
        self._alerts.append(alert)
        return alert["id"]

    def send_with_alert_id(self, alert_id, status, meta, kwargs):
        """ Send a status message on the given alert with the corresponding meta/kwarg arguments
            Returns True if alert was send else False"""
        assert status in [Status.ERROR, Status.OK, Status.OFFLINE]
        _status_dict = {
            "status": status,
            "alert_meta": (meta or {}),
        }
        _status_dict.update(kwargs or {})
        return self._send_message(alert_id, _status_dict)

    def send(self, source, alert_type, handle_type, package_name=None, version=None,
             always_send_alert=None, status=Status.ERROR, meta=None, kwargs=None):
        """Sends an alert to the device serve

        Args:
            source (str): The source of the error (i.e. which pin, camera)
            alert_type (str): Name of the alert all lower_case with '_' (i.e. high_time_check, health_hw_compute_nuc)
            handle_type (str): How the error should be handled on the server (state, state_change, count)
            status (Status/bool): The status you want to send(Status.OK/True, Status.ERROR/False, Status.OFFLINE)
            package_name (str) optional: You can send errors for other packages
            version (str) (default=config.VERSION): Current version of the package
            always_send_alert (bool): Flag to declare if you want to send every alert or only every change in the status
            meta (dict): Meta information
            kwargs (dict): Additional parameters you want to add to the message (i.e. value of error metric)

        Returns:
            True if alert was send else False
        """
        assert "-" not in alert_type, "please do not use - in your alert type names"

        alert_id = self.get_or_add_alert_id(source, alert_type, handle_type, package_name, version, always_send_alert)
        return self.send_with_alert_id(alert_id, status, meta, kwargs)



import codecs
import datetime
import os
import re
import subprocess
import threading
import time

from queue import Queue, Full, Empty


class LogRotator(object):
    def __init__(self, directory, prefix="hour_", flush_interval=60.0, testmode=False, binary_write=False):
        self.directory = directory
        try:
            os.makedirs(self.directory)
        except:
            pass
        self.prefix = prefix
        self.testmode = testmode
        self.binary_write = binary_write
        self.file = None
        self.file_hour = None
        self.current_filename = None
        self.rotation_time = 3600
        self.flush_interval = flush_interval
        self.last_flush_time = 9999999999.9
        self.gzip_timestamp = time.time() + self.rotation_time / 60
        self.queue = None
        self.thread = None

    def get_hour(self):
        return int(time.time() / self.rotation_time)

    def filename(self, hour):
        if self.testmode:
            return "log.log"
        else:
            return "{}{}.log".format(
                self.prefix, datetime.datetime.utcfromtimestamp(hour*self.rotation_time).strftime("%Y%m%d%H%M%S")
            )

    def filepath(self, hour):
        return os.path.join(self.directory, self.filename(hour))

    def _open_file(self):
        try:
            if self.file:
                self.file.close()
        except:
            pass

        self.file_hour = self.get_hour()
        if self.binary_write:
            self.file = open(self.filepath(self.file_hour), "ab")
        else:
            self.file = codecs.open(self.filepath(self.file_hour), "a", encoding="utf8")
        self.current_filename = self.filename(self.file_hour)
        self.last_flush_time = time.time()
        self.gzip_timestamp = time.time() + self.rotation_time / 60

    def gzip(self):
        for file in os.listdir(self.directory):
            if file.endswith(".log") and re.match(self.prefix + "\d+\.log",
                                                  file) and file != self.current_filename and not os.path.exists(
                    os.path.join(self.directory, file + ".gz")):
                subprocess.call("gzip {}".format(os.path.join(self.directory, file)), shell=True)

    def logrotate(self):
        if self.file:
            new_hour = self.get_hour()
            if new_hour != self.file_hour:
                self._open_file()
        else:
            self._open_file()

        if time.time() > self.gzip_timestamp:
            self.gzip()
            self.gzip_timestamp += self.rotation_time

    def write(self, text, flush=False):
        self.logrotate()
        self.file.write(text)
        if flush or time.time() - self.last_flush_time > self.flush_interval:
            self.flush()

    def flush(self):
        if self.file:
            self.last_flush_time = time.time()
            self.file.flush()

    def run(self):
        while True:
            try:
                self.write(self.queue.get(timeout=max(0.0, self.last_flush_time+self.flush_interval-time.time())))
            except Empty:
                self.flush()

    def start_as_thread(self):
        if not self.queue:
            self.queue = Queue(maxsize=1000)
        if not self.thread:
            self.thread = threading.Thread(target=self.run)
            self.thread.setDaemon(True)
            self.thread.start()

    def put_on_queue(self, line):
        if self.queue:
            try:
                self.queue.put_nowait(line)
            except Full:
                return "Full"
        else:
            return "Not running"
        return ""



from __future__ import absolute_import

import logging
import re
from threading import RLock

import sentry_sdk
import zmq
from om.message import Message, Topic
from sentry_sdk.transport import Transport

dsn_regex = re.compile(r'([a-z0-9]+):\/\/([a-z0-9:]+)@([a-z0-9\.]+)\/(\d+)')


class ZeroMQTransport(Transport):
    def __init__(self, config, options=None):
        super(ZeroMQTransport, self).__init__(options)
        context = zmq.Context()
        self.pub = context.socket(zmq.PUB)
        self.pub.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_SUB_PORT))
        self.name = config.NAME
        self.lock = RLock()

    def capture_event(self, event):
        message = Message(name=self.name, type="sentry")
        message.update(event=event)
        with self.lock:
            message.to_socket(self.pub, Topic.SENTRY)

    def flush(self, timeout, callback=None):
        pass

    def kill(self):
        pass


def sentry_init(config, extras=None, **kwargs):
    """
    This function initializes Sentry with our predefined values. This function also check if Sentry should be
    initialized.

    :param config: current config
    :param extras: global extras that should be added to message
    :param kwargs: client supported **kwargs see ClientOptions
    """
    if activate_sentry(config):
        if not extras:
            extras = {}
        extras.update(_extra_from_config(config))

        transport = ZeroMQTransport(config)
        environment = environment_from_config(config)
        sentry_sdk.init(
            None, release=config.VERSION, server_name=config.RESOURCE, environment=environment, transport=transport,
            **kwargs
        )
        with sentry_sdk.configure_scope() as scope:
            for key, value in extras.items():
                scope.set_extra(key, value)
            scope.user = {"username": config.RESOURCE}


def activate_sentry(config):
    """
    This function checks if for the given config Sentry should be activated or not.

    :param config: current configuration
    :return: bool
    """
    if config.VERSION == "unknown":
        logging.info("not activating sentry because version is unknown")
        return False
    return True


def parse_dsn(dsn):
    """
    This function parses a given DSN into it's components.

    :param dsn: The DSN from sentry
    :return: dictionary with values if DSN could be matched against the regex, if the given DSN is deprecated then the
    secret is in the 'public_key' field
    """
    if dsn:
        matches = dsn_regex.search(dsn)
        if matches:
            return {
                "protocol": matches.group(1),
                "public_key": matches.group(2),
                "host": matches.group(3),
                "project_id": matches.group(4),
            }
    return {}


def environment_from_config(config):
    """
    This functions returns either the environment of the current configuration.

    :param config: current configuration
    :return: 'staging' or 'production'
    """
    if hasattr(config, "DEVICE_HOST") and ("test" in config.DEVICE_HOST):
        return "staging"
    else:
        return "production"


def _extra_from_config(config):
    extra = {
        "name": config.NAME,
    }
    if hasattr(config, "GATE"):
        extra["gate"] = config.GATE
    return extra



import logging
import os
import sys
import threading
import time
import traceback
from collections import namedtuple
from uuid import uuid4

import sentry_sdk
import zmq

from om.message import Message

CallbackEntry = namedtuple("CallbackEntry", ["timestamp", "result"])


def channel_to_response_topic(channel):
    if hasattr(channel, "decode"):
        cpy = channel.decode("utf-8")
    else:
        cpy = channel
    topic = "rpc-rep-{}".format(cpy)
    if hasattr(topic, "encode"):
        return topic.encode("ascii")
    return topic


def channel_to_request_topic(channel):
    if hasattr(channel, "decode"):
        cpy = channel.decode("utf-8")
    else:
        cpy = channel
    topic = "rpc-req-{}".format(cpy)
    if hasattr(topic, "encode"):
        return topic.encode("ascii")
    return topic


class RPCClient(object):
    class RPCEntry(object):
        def __init__(self):
            self.timestamp = time.time()
            self.result = None

    def __init__(self, config, context=None, channels=None):
        if channels is None:
            channels = []
        context = context or zmq.Context()
        self.log = logging.getLogger("rcp-client")
        self.lock = threading.RLock()
        self.results = dict()
        self.name = config.NAME
        if hasattr(config, "RPC_CLIENT_SYNC_TIMEOUT"):
            self.sync_timeout = config.RPC_CLIENT_SYNC_TIMEOUT
        else:
            self.sync_timeout = 3.0
        self.sub = context.socket(zmq.SUB)
        self.sub.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_PUB_PORT))
        self.channels = channels
        for channel in channels:
            topic = channel_to_response_topic(channel)
            self.log.info("subscribing to channel: {}".format(topic))
            self.sub.subscribe(topic)
        self.pub = context.socket(zmq.PUB)
        self.pub.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_SUB_PORT))
        self.receiver = threading.Thread(target=self.recv_thread)
        self.receiver.setDaemon(True)
        self.receiver.start()

    def return_error(self, status):
        return {"status": status}

    def return_success(self, message):
        response = message.get("response", {})
        message.setdefault("status", "success")
        if isinstance(response, dict):
            message["status"] = response.get("status")
            response.pop("status", None)
        return message

    def cleanup_old_results(self):
        now = time.time()
        with self.lock:
            to_delete = []
            for rpc_id, entry in self.results.items():
                if now - entry.timestamp > 10:
                    to_delete.append(rpc_id)
            for rpc_id in to_delete:
                try:
                    del self.results[rpc_id]
                except KeyError:
                    pass

    def _call(self, channel, type, request):
        rpc_id = str(uuid4())
        message = Message(
            name=self.name,
            type=type,
            rpc_id=rpc_id,
            request=request,
        )
        message.to_socket(self.pub, channel_to_request_topic(channel))
        return rpc_id

    def call_async(self, channel, type, request):
        return self._call(channel, type, request)

    def call_sync(self, channel, type, request):
        self.log.warning("running in sync mode, this may cause delay")
        rpc_id = self.call_async(channel, type, request)
        if channel not in self.channels:
            self.log.error("called in sync mode, but no subscription on channel is present!", extra={
                "channel": channel,
                "subscriptions": str(self.channels)
            })
            return self.return_error("no_subscription_on_channel")
        with self.lock:
            self.results[rpc_id] = self.RPCEntry()
        # wait a little because service cannot answer instantly
        time.sleep(0.1)
        while True:
            with self.lock:
                entry = self.results.get(rpc_id)
            if entry.result:
                with self.lock:
                    self.results.pop(rpc_id, None)
                return self.return_success(entry.result)
            duration = (time.time() - entry.timestamp)
            if duration > self.sync_timeout:
                return self.return_error("timeout")
            time.sleep(self.sync_timeout / 5.0)

    def rpc(self, channel, type, request):
        result = self.call_sync(channel, type, request)
        return result.get("response") or {}

    def recv_thread(self):
        try:
            while True:
                topic, message = Message.from_socket(self.sub)
                rpc_id = message.get("rpc_id")
                with self.lock:
                    if rpc_id in self.results:
                        self.results[rpc_id].result = message
        except KeyboardInterrupt:
            pass
        except Exception as e:
            sentry_sdk.capture_exception(e)
            traceback.print_exc(file=sys.stderr)
            sys.stderr.flush()
            time.sleep(1)
            os._exit(1)


class RPCServer(object):
    @staticmethod
    def gateway_filter(gate=None, direction=None):
        def _filter(request, message, handler):
            gateway = request.get("gateway")
            if not gateway:
                return False
            return (not gate or gate == gateway.get("gate")) and \
                   (not direction or direction == gateway.get("direction"))

        return _filter

    def __init__(self, config, context=None):
        context = context or zmq.Context()
        self.name = config.NAME
        self.sub = context.socket(zmq.SUB)
        self.sub.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_PUB_PORT))
        self.pub = context.socket(zmq.PUB)
        self.pub.connect("tcp://{}:{}".format(config.BROKER_HOST, config.BROKER_SUB_PORT))
        self.handlers = {}
        self.filters = []
        self.log = logging.getLogger("rpc")

    def add_filter(self, handler):
        self.filters.append(handler)

    def _thread_wrapper(self):
        try:
            self.run()
        except KeyboardInterrupt:
            pass
        except Exception as e:
            sentry_sdk.capture_exception()
            traceback.print_exc(file=sys.stderr)
            sys.stderr.flush()
            time.sleep(1)
            os._exit(1)

    def register_handler(self, channel, type, handler):
        """
        :param type: the request type, specify * to catch all types, in this case no other
                     handler will be run for the channel except for the wildcard handler
        :param handler: request handler of the form (request:dict, meta:dict) -> any
                        if the handler returns a dict which contains the key "status",
                        then the value of "status" will be used as the rpc response status.
                        There are no other reserved keys except "status".
        """
        self.log.debug("register handler {}:{} -> {}".format(channel, type, handler))
        channel = self._ascii(channel)
        type = self._ascii(type)
        self.sub.subscribe(channel_to_request_topic(channel))
        self.handlers[(channel, type)] = handler

    def _channel_from_topic(self, topic):
        return topic.split(b"-", 2)[-1]

    def _ascii(self, x):
        if hasattr(x, "encode"):
            return x.encode("ascii")
        else:
            return x

    def _handlers_handler(self):
        pass

    def should_process_message(self, request, message, handler):
        if self.filters:
            for filter in self.filters:
                if filter(request=request, message=message, handler=handler):
                    return True
            return False
        return True

    def find_handler(self, channel, type):
        channel = self._ascii(channel)
        type = self._ascii(type)
        if type == "_handlers":
            return self._handlers_handler
        else:
            return self.handlers.get((channel, b"*")) or self.handlers.get((channel, type))

    def run_as_thread(self):
        thread = threading.Thread(target=self._thread_wrapper)
        thread.setDaemon(True)
        thread.start()
        return thread

    def run(self):
        while True:
            topic, message = Message.from_socket(self.sub)
            channel = self._channel_from_topic(topic)
            request = message["request"]
            type = message["type"]
            del message["request"]
            message["channel"] = channel

            self.log.debug("processing request for {}:{} -> {}".format(topic, channel, request))
            handler = self.find_handler(channel, type)
            if handler:
                try:
                    if not self.should_process_message(request, message, handler):
                        continue
                except Exception as e:
                    sentry_sdk.capture_exception()
                    self.log.info("exception in filter {}:{}".format(channel, type))
                    continue

                try:
                    response = handler(request, message)
                except Exception as e:
                    sentry_sdk.capture_exception()
                    self.log.info("exception in handler {}:{}".format(channel, type))
                    response = {"status": "handler_error", "exception": str(e)}
            else:
                self.log.warning("no handler found for {}:{}".format(channel, type))
                continue

            self.log.debug("response: {}".format(response))
            result = Message(type=type, name=self.name)
            result["rpc_id"] = message["rpc_id"]
            result["response"] = response
            result.to_socket(self.pub, channel_to_response_topic(channel))



import logging

from .config import log_level as __log_level
from .healthz import Healthz, HealthzStatus


def setup_logging():
    log_level = __log_level()
    logging.basicConfig(
        level=log_level,
        format="%(asctime)s:%(levelname)s:%(name)s -> %(message)s [%(filename)s:%(lineno)s]"
    )


__all__ = ['Healthz', 'setup_logging']
